/*
 *************************************************************************************************
 * boards.c
 *	Register and Find Boards
 *
 * Copyright 2005 Jochen Karrer. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice, this list of
 *       conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright notice, this list
 *       of conditions and the following disclaimer in the documentation and/or other materials
 *       provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY Jochen Karrer ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are those of the
 * authors and should not be interpreted as representing official policies, either expressed
 * or implied, of Jochen Karrer.
 *
 *************************************************************************************************
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include "boards.h"
#include "ns9750dev.h"
#include "lacc.h"
#include "facc.h"
#include "uzebox.h"
#include "alt9.h"
#include "m32cfant.h"
#include "armee.h"
#include "nxdb500.h"
#include "fbdd.h"
#include "unc90.h"
#include "stk500.h"
#include "nbdd.h"
#include "imx21ads.h"

static Board *first_board = NULL;

Board * 
Board_Find(const char *name) 
{
	Board *cursor;
	for(cursor=first_board;cursor;cursor=cursor->next) {
		if(!strcmp(name,cursor->name)) {
			return cursor;
		}
	}	
	fprintf(stderr,"Board \"%s\" does not exist.\nList of available boards:\n",name);
	for(cursor=first_board;cursor;cursor=cursor->next) {
		fprintf(stderr,"Board %-15s: %s\n",cursor->name,cursor->description);
	}	
	return NULL;
}

void
Board_Register(Board *board) 
{
	board->next=first_board;
	first_board=board;
}

void
Board_Create(Board *board)  {
	board->createBoard();
}
